#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#ifdef _OPENMP
	#include <omp.h>
#endif
#ifdef _MPI
	#include <mpi.h>
#endif
#include "lis.h"

int main(int argc, char* argv[])
{
	LIS_MATRIX		A;
	LIS_VECTOR		b,x;

	int				nprocs,my_rank;
	int				i,j,ii,jj,k,n,m,nn;
	int				is,ie;
	int				err;
	int				options[LIS_OPTIONS_LEN];
	LIS_SCALAR		params[LIS_PARAMS_LEN];

	lis_initialize(argc, argv, NULL, options, params);

	if( argc < 4 )
	{
		printf("test2 m n matrix_filename \n");
		lis_finalize();
		exit(0);
	}
	n  = atoi(argv[1]);
	m  = atoi(argv[2]);
	if( n<=0 || m<=0 )
	{
		printf("n=%d <=0 or m=%d <=0\n",n,m);
		lis_finalize();
		exit(0);
	}

#ifdef _MPI
    MPI_Comm_size(MPI_COMM_WORLD,&nprocs);
    MPI_Comm_rank(MPI_COMM_WORLD,&my_rank);
#else
	nprocs  = 1;
	my_rank = 0;
#endif

#ifdef _OPENMP
	printf("Max Procs   = %d\n",omp_get_num_procs());
	printf("Max Threads = %d\n",omp_get_max_threads());
#endif

	nn = n*m;
	lis_matrix_create(0,nn,LIS_COMM_WORLD,&A);

	lis_matrix_get_range(A,&is,&ie);
	for(ii=is;ii<ie;ii++)
	{
		i = ii/n;
		j = ii - i*n;
		if( i>0 )   { jj = ii - n; lis_matrix_set(ii,jj,-1.0,A);}
		if( i<m-1 ) { jj = ii + n; lis_matrix_set(ii,jj,-1.0,A);}
		if( j>0 )   { jj = ii - 1; lis_matrix_set(ii,jj,-1.0,A);}
		if( j<n-1 ) { jj = ii + 1; lis_matrix_set(ii,jj,-1.0,A);}
		lis_matrix_set(ii,ii,4.0,A);
	}
	lis_matrix_assemble(&A,LIS_MATRIX_CRS);
	lis_vector_create(0,nn,LIS_COMM_WORLD,&b);
	lis_vector_duplicate(b,&x);
	lis_vector_set_all(1.0,x);
	lis_matvec(A,x,b);

	lis_output(A,b,NULL,LIS_FMT_MM,argv[3]);

	lis_matrix_destroy(A);
	lis_vector_destroy(b);
	lis_vector_destroy(x);

	lis_finalize();
	return 0;
}
